﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinTree;

namespace UltraTreeWithCheckBoxColumns
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            SetupTree();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void SetupTree()
        {
            // Set up the UltraCheckEditor so it looks good in a cell. 
            ultraCheckEditor1.Text = string.Empty;
            ultraCheckEditor1.CheckAlign = ContentAlignment.MiddleCenter;

            UltraTreeColumnSet commonColumnSet = ultraTree.ColumnSettings.ColumnSets["columnSet"];
            ultraTree.ColumnSettings.RootColumnSet = commonColumnSet;

            // Populate the tree with nodes
            UltraTreeNode nodeA = ultraTree.Nodes.Add("A");
            UltraTreeNode nodeB = ultraTree.Nodes.Add("B");
            nodeA.Cells["Name"].Value = nodeA.Key;
            nodeB.Cells["Name"].Value = nodeB.Key;
            nodeA.Override.ColumnSet = commonColumnSet;
            nodeB.Override.ColumnSet = commonColumnSet;

            foreach (var n in ultraTree.Nodes)
            {
                UltraTreeNode node1 = n.Nodes.Add(n.Key + " 1");
                UltraTreeNode node2 = n.Nodes.Add(n.Key + " 2");
                node1.Cells["Name"].Value = node1.Key;
                node2.Cells["Name"].Value = node2.Key;
                node1.Override.ColumnSet = commonColumnSet;
                node2.Override.ColumnSet = commonColumnSet;

                foreach (var m in n.Nodes)
                {
                    for (int i = 1; i < 5; i++)
                    {
                        UltraTreeNode leaf = m.Nodes.Add(m.Key + ": " + i + " leaf");
                        leaf.Cells["Name"].Value = leaf.Key;
                        leaf.Cells["CheckBox"].Value = i % 3 == 0;
                        leaf.Override.ColumnSet = commonColumnSet;
                    }
                }
            }

            commonColumnSet.Columns["Name"].PerformAutoResize(ColumnAutoSizeMode.AllNodesWithDescendants);
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void ultraTree_CellValueChanged(object sender, CellValueChangedEventArgs e)
        {
            UltraTreeNodeCell cell = e.Cell;
            if (cell == null)
                return;

            UltraTreeNode node = cell.Node;
            string key = cell.Key;

            SetChildNodes(node, key, (bool)e.CurrentValue);
            SetParentNodes(node, key);
        }

        private void SetChildNodes(UltraTreeNode node, string columnKey, bool set)
        {
            node.Cells[columnKey].Value = set;

            foreach (var n in node.Nodes)
                SetChildNodes(n, columnKey, set);
        }

        private void SetParentNodes(UltraTreeNode node, string columnKey)
        {
            if (node.Parent == null)
                return;

            int checkedNodes = 0;
            int uncheckedNodes = 0;
            bool indeterminate = false;

            UltraTreeNode parent = node.Parent;
            TreeNodesCollection collection = node.ParentNodesCollection;
            foreach (var n in collection)
            {
                if (n.Cells[columnKey].Value == null)
                {
                    indeterminate = true;
                    break;
                }

                if ((bool)n.Cells[columnKey].Value)
                    checkedNodes++;
                else
                    uncheckedNodes++;
            }

            if (indeterminate || (checkedNodes > 0 && uncheckedNodes > 0))
                parent.Cells[columnKey].Value = null;
            else if (checkedNodes > 0)
                parent.Cells[columnKey].Value = true;
            else
                parent.Cells[columnKey].Value = false;

            SetParentNodes(parent, columnKey);
        }
    }
}
